package jp.sbcr.java_framework.web.log.filter;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import jp.sbcr.java_framework.web.log.AccessLogRecord;
import jp.sbcr.java_framework.web.log.service.AccessLogService;

import org.seasar.framework.container.S2Container;
import org.seasar.framework.container.factory.S2ContainerFactory;

public class AccessLogFilter implements Filter {

	private AccessLogService service;

	public void init(FilterConfig config)
			throws ServletException {

		S2Container container = S2ContainerFactory
				.create("AccessLogService.dicon");
		container.init();
		service = (AccessLogService) container
				.getComponent(AccessLogService.class);
	}

	public void doFilter(ServletRequest request,
			ServletResponse response, FilterChain chain)
			throws IOException, ServletException {

		doFilter((HttpServletRequest) request,
				(HttpServletResponse) response, chain);

	}

	private void doFilter(HttpServletRequest request,
			HttpServletResponse response, FilterChain chain)
			throws IOException, ServletException {

		AccessLogRecord record = null;
		try {
			record = createAccessLogRecord(request);
			service.loggingRequest(record);

			chain.doFilter(request, response);
		} finally {
			setResponseValue(record, response);
			service.loggingResponse(record);
		}
	}

	private void setResponseValue(AccessLogRecord record,
			HttpServletResponse response) {

		record.setResponseContentType(response.getContentType());
		record.setResponseLocale(toString(response.getLocale()));
	}

	private String toString(Locale locale) {
		if (locale == null) {
			return null;
		}
		return locale.toString();
	}

	private AccessLogRecord createAccessLogRecord(
			HttpServletRequest request) {

		AccessLogRecord accessLog = new AccessLogRecord();

		accessLog.setContextPath(request.getContextPath());
		accessLog.setHeaders(getHeaderMap(request).toString());
		accessLog.setMethod(request.getMethod());
		accessLog.setPathInfo(request.getPathInfo());
		accessLog.setPathTranslated(request.getPathTranslated());
		accessLog.setQueryString(request.getQueryString());
		accessLog.setRemoteUser(request.getRemoteUser());
		accessLog.setRequestedSessionId(request
				.getRequestedSessionId());
		accessLog.setRequestUri(request.getRequestURI());
		accessLog.setRequestUrl(request.getRequestURL()
				.toString());
		accessLog.setServletPath(request.getServletPath());
		accessLog.setContentType(request.getContentType());
		accessLog.setLocalAddr(request.getLocalAddr());
		accessLog.setLocales(getLocaleList(request).toString());
		accessLog.setLocalName(request.getLocalName());
		accessLog.setLocalPort(request.getLocalPort());
		accessLog.setParameters(getParameterMap(request)
				.toString());
		accessLog.setProtocol(request.getProtocol());
		accessLog.setRemoteAddr(request.getRemoteAddr());
		accessLog.setRemoteHost(request.getRemoteHost());
		accessLog.setScheme(request.getScheme());
		accessLog.setServerName(request.getServerName());

		return accessLog;
	}

	private Map<String, List<String>> getParameterMap(
			HttpServletRequest request) {

		Map<String, List<String>> map = new HashMap<String, List<String>>();
		@SuppressWarnings("unchecked")
		Enumeration<String> paramNames = request
				.getParameterNames();
		while (paramNames.hasMoreElements()) {
			String name = paramNames.nextElement();
			String[] values = request.getParameterValues(name);
			if (values == null) {
				map.put(name, new ArrayList<String>());
			} else {
				map.put(name, Arrays.asList(values));
			}
		}
		return map;
	}

	private List<String> getLocaleList(HttpServletRequest request) {

		List<String> list = new ArrayList<String>();
		@SuppressWarnings("unchecked")
		Enumeration<Locale> locales = request.getLocales();
		while (locales.hasMoreElements()) {
			Locale locale = locales.nextElement();
			list.add(locale.toString());
		}
		return list;
	}

	private Map<String, String> getHeaderMap(
			HttpServletRequest request) {

		Map<String, String> map = new HashMap<String, String>();
		@SuppressWarnings("unchecked")
		Enumeration<String> headerNames = request
				.getHeaderNames();
		while (headerNames.hasMoreElements()) {
			String headerName = headerNames.nextElement();
			map.put(headerName, request.getHeader(headerName));
		}
		return map;
	}

	public void destroy() {
	}

}
