package jp.sbcr.java_framework.validator;

import java.util.Iterator;

import javax.servlet.http.HttpServletRequest;

import org.apache.struts.Globals;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;
import org.apache.struts.util.MessageResources;
import org.apache.struts.util.MessageResourcesFactory;
import org.seasar.extension.unit.S2TestCase;

public class JChecksTest extends S2TestCase {

	private HttpServletRequest request;

	@Override
	protected void setUp() throws Exception {
		super.setUp();

		request = getRequest();
		MessageResourcesFactory factory = MessageResourcesFactory
				.createFactory();
		MessageResources resources = factory
				.createResources("config");
		request.setAttribute(Globals.MESSAGES_KEY, resources);
	}

	@Override
	protected void tearDown() throws Exception {
		super.tearDown();
	}

	public void testValidate全角ひらがなが有効な場合() {
		TestForm form = new TestForm();
		form.setFullHiragana("「あいうえお。」");

		ActionMessages errors = new ActionMessages();

		JChecks.validate(form, errors, request);

		assertEquals(0, errors.size());
	}

	public void testValidateシリアル番号が有効な場合() {
		TestForm form = new TestForm();
		form.setSerialNo("ABC123456XYZ");

		ActionMessages errors = new ActionMessages();
		JChecks.validate(form, errors, request);
		assertEquals(0, errors.size());
	}

	public void testValidate全角ひらがなががエラーの場合() {
		TestForm form = new TestForm();
		form.setFullHiragana("「アイウエオ。」");

		ActionMessages errors = new ActionMessages();

		JChecks.validate(form, errors, request);

		assertEquals(1, errors.size());

		@SuppressWarnings("unchecked")
		Iterator<ActionMessage> iterator = errors
				.get("fullHiragana");
		assertTrue(iterator.hasNext());
		ActionMessage message = iterator.next();
		assertEquals("errors.full.hiragana", message.getKey());
		assertTrue(message.isResource());
	}

	public void testValidateシリアル番号がエラーの場合() {
		TestForm form = new TestForm();
		form.setSerialNo("XYZ123456ABC");

		ActionMessages errors = new ActionMessages();
		JChecks.validate(form, errors, request);
		assertEquals(1, errors.size());

		@SuppressWarnings("unchecked")
		Iterator<ActionMessage> iterator = errors
				.get("serialNo");
		assertTrue(iterator.hasNext());
		ActionMessage message = iterator.next();
		assertEquals("errors.serial.no", message.getKey());
		assertTrue(message.isResource());
	}

	public void testValidate複数プロパティーを同時にチェックする() {
		TestForm form = new TestForm();
		form.setFullHiragana("「アイウエオ。」");
		form.setSerialNo("XYZ123456ABC");
		form.setEmptyValue("");

		ActionMessages errors = new ActionMessages();

		JChecks.validate(form, errors, request);

		assertEquals(2, errors.size());

		@SuppressWarnings("unchecked")
		Iterator<ActionMessage> iterator = errors
				.get("fullHiragana");
		assertTrue(iterator.hasNext());
		ActionMessage message = iterator.next();
		assertEquals("errors.full.hiragana", message.getKey());
		assertTrue(message.isResource());
		assertEquals("全角ひらがな", message.getValues()[0]);

		@SuppressWarnings("unchecked")
		Iterator<ActionMessage> iterator2 = errors
				.get("serialNo");
		assertTrue(iterator2.hasNext());
		ActionMessage message2 = iterator2.next();
		assertEquals("errors.serial.no", message2.getKey());
		assertTrue(message2.isResource());
		assertEquals("シリアル番号", message2.getValues()[0]);
	}

}
